package fr.eisti.ing1.java.graphe;

import java.util.*;

public class GrapheValue<T extends Comparable<T>> {

	private int nombreSommets;
	private Set<AreteValuee<T>> aretes;

	public GrapheValue(int nombreSommets) {
		this.nombreSommets = nombreSommets;
		this.aretes = new HashSet<AreteValuee<T>>();
	}

	public boolean ajouterArete(AreteValuee<T> arete) {
		if ((arete.getOrigine() < 1) 
				|| (arete.getOrigine() > nombreSommets)
				|| (arete.getDestination() <1)
				|| (arete.getDestination() > nombreSommets)) {
			return false;
		}
		return aretes.add(arete);
	}

	public GrapheValue<T> arbreMinimal() {
		GrapheValue<T> arbre;	// arbre couvrant minimal du graphe
		List<AreteValuee<T>> aretesOrdonnees;
		int[] composantesConnexes; // tableau des composantes connexes
		int sommet1, sommet2;

		// création de l'arbre minimal (sommets)
		arbre = new GrapheValue<T>(this.nombreSommets);

		// tri des aretes valuées
		aretesOrdonnees = new ArrayList<AreteValuee<T>>(nombreSommets);
		aretesOrdonnees.addAll(this.aretes);
		Collections.sort(aretesOrdonnees);

		// initialisation des composantes connexes
		composantesConnexes = new int[this.nombreSommets];
		// la case i du tableau représente le sommet i+1
		for (int i=0; i<nombreSommets; i++) {
			composantesConnexes[i] = i+1;
		}

		// algorithme de Kruskal
		for (AreteValuee<T> arete : aretesOrdonnees) {
			sommet1 = arete.getOrigine(); 
			sommet2 = arete.getDestination(); 
			if (composantesConnexes[sommet1-1] !=
					composantesConnexes[sommet2-1]) {
				arbre.ajouterArete(new AreteValuee<T>(sommet1, sommet2, 
						arete.getValeur()));
				majComposantesConnexes(composantesConnexes, sommet1, sommet2);
			}
		}
		return arbre;
	}

	private	void majComposantesConnexes(int[] composantesConnexes, int sommet1,
			int sommet2) {
		int numSommet, maxCC, minCC, taille;
		
		if (composantesConnexes[sommet1-1] > composantesConnexes[sommet2-1]) {
			maxCC = composantesConnexes[sommet1-1];
			minCC = composantesConnexes[sommet2-1];
		} else  {
			maxCC = composantesConnexes[sommet2-1];
			minCC = composantesConnexes[sommet1-1];
		}
		for (numSommet=1; numSommet<=composantesConnexes.length; numSommet++) {
			if (composantesConnexes[numSommet-1] == maxCC) {
				composantesConnexes[numSommet-1] = minCC;
			}
		}
	}
}

